==> vueform-1.5.5.tgz: package/src/services/validation/factory.js <==
import _ from 'lodash'
import parse from './parse'
import replaceWildcards from './../../utils/replaceWildcards'
import compare from './../../utils/compare'
 
const Factory = class {

  constructor(path, form$) {
    this.form$ = form$
    this.element$ = form$.el$(path)
  }

  get rules() {
    return Object.assign({}, this.form$.$vueform.services.validation.rules, this.form$.$vueform.rules)
  }
 
  makeAll(rules) {
    let parsedRules = this.parseRules(rules)

    if (parsedRules.length == 0) {
      return []
    }

    return _.map(parsedRules, (rule) => {
      return this.make(rule)
    })
  }
  
  make(rule) {
    let ruleClass = typeof rule == 'function' ? rule : this.rules[rule.name]

    if (!ruleClass) {
      throw new Error(`Unknown rule: '${rule.name}'`)
    }

    return new ruleClass(rule, {
      element$: this.element$
    })
  }

  parseRules(rules) {
    if (!_.isArray(rules)) {
      rules = rules.split('|')
    }
    
    return _.flatMap(rules, (rule) => {
      if (typeof rule == 'function') {
        return rule
      }

      return this.isConditional(rule) ? this.parseConditional(rule) : this.parse(rule)
    })
  }

  parse(rule) {
    return parse(rule)
  }

  isConditional(rule) {
    return _.isPlainObject(rule)
  }

  parseConditional(rule) {
    let conditions = _.values(rule)[0]

    if (!Array.isArray(conditions[0])) {
      conditions = [conditions]
    }

    let parsed = {
      ...parse(_.keys(rule)[0]),
      conditions: (form$, Validator, el$) => {
        return conditions.every((condition) => {
          if (_.isArray(condition)) {
            if (_.isArray(condition[0])) {
              return condition.some((subcondition) => {
                if (_.isArray(subcondition)) {
                  return this.createConditionFromArray(subcondition)(form$, Validator, el$)
                } else {
                  return condition(form$, Validator, el$)
                }
              })
            } else {
              return this.createConditionFromArray(condition)(form$, Validator, el$)
            }
          } else {
            return condition(form$, Validator, el$)
          }
        })
      },
      dependents: [],
    }

    conditions.forEach((condition) => {
      if (_.isArray(condition)) {
        if (_.isArray(condition[0])) {
          condition.forEach((subcondition) => {
            if (_.isArray(subcondition)) {
              parsed.dependents.push(replaceWildcards(subcondition[0], this.element$.path))
            }
          })
        } else {
          parsed.dependents.push(replaceWildcards(condition[0], this.element$.path))
        }
      }
    })

    return parsed
  } 

  createConditionFromArray(condition) {
    let field = replaceWildcards(condition[0], this.element$.path)
    let operator = condition.length == 3 || ['empty', 'not_empty', 'today'].indexOf(condition[1]) !== -1 ? condition[1] : '=='
    let value = condition.length == 3 ? condition[2] : (
      ['empty', 'not_empty', 'today'].indexOf(condition[1]) === -1 ? condition[1] : true
    )

    return (form$, Validator, el$) => {
      var actual = _.get(form$.requestData, field)
      var expected = value

      return compare(actual, operator, expected, this.element$)
    }
  }
}
 
export default Factory

==> vueform-1.5.7.tgz: package/src/services/validation/factory.js <==
import map from 'lodash-es/map'
import isArray from 'lodash-es/isArray'
import flatMap from 'lodash-es/flatMap'
import isPlainObject from 'lodash-es/isPlainObject'
import values from 'lodash-es/values'
import keys from 'lodash-es/keys'
import get from 'lodash-es/get'
import parse from './parse'
import replaceWildcards from './../../utils/replaceWildcards'
import compare from './../../utils/compare'
 
const Factory = class {

  constructor(path, form$) {
    this.form$ = form$
    this.element$ = form$.el$(path)
  }

  get rules() {
    return Object.assign({}, this.form$.$vueform.services.validation.rules, this.form$.$vueform.rules)
  }
 
  makeAll(rules) {
    let parsedRules = this.parseRules(rules)

    if (parsedRules.length == 0) {
      return []
    }

    return map(parsedRules, (rule) => {
      return this.make(rule)
    })
  }
  
  make(rule) {
    let ruleClass = typeof rule == 'function' ? rule : this.rules[rule.name]

    if (!ruleClass) {
      throw new Error(`Unknown rule: '${rule.name}'`)
    }

    return new ruleClass(rule, {
      element$: this.element$
    })
  }

  parseRules(rules) {
    if (!isArray(rules)) {
      rules = rules.split('|')
    }
    
    return flatMap(rules, (rule) => {
      if (typeof rule == 'function') {
        return rule
      }

      return this.isConditional(rule) ? this.parseConditional(rule) : this.parse(rule)
    })
  }

  parse(rule) {
    return parse(rule)
  }

  isConditional(rule) {
    return isPlainObject(rule)
  }

  parseConditional(rule) {
    let conditions = values(rule)[0]

    if (!Array.isArray(conditions[0])) {
      conditions = [conditions]
    }

    let parsed = {
      ...parse(keys(rule)[0]),
      conditions: (form$, Validator, el$) => {
        return conditions.every((condition) => {
          if (isArray(condition)) {
            if (isArray(condition[0])) {
              return condition.some((subcondition) => {
                if (isArray(subcondition)) {
                  return this.createConditionFromArray(subcondition)(form$, Validator, el$)
                } else {
                  return condition(form$, Validator, el$)
                }
              })
            } else {
              return this.createConditionFromArray(condition)(form$, Validator, el$)
            }
          } else {
            return condition(form$, Validator, el$)
          }
        })
      },
      dependents: [],
    }

    conditions.forEach((condition) => {
      if (isArray(condition)) {
        if (isArray(condition[0])) {
          condition.forEach((subcondition) => {
            if (isArray(subcondition)) {
              parsed.dependents.push(replaceWildcards(subcondition[0], this.element$.path))
            }
          })
        } else {
          parsed.dependents.push(replaceWildcards(condition[0], this.element$.path))
        }
      }
    })

    return parsed
  } 

  createConditionFromArray(condition) {
    let field = replaceWildcards(condition[0], this.element$.path)
    let operator = condition.length == 3 || ['empty', 'not_empty', 'today'].indexOf(condition[1]) !== -1 ? condition[1] : '=='
    let value = condition.length == 3 ? condition[2] : (
      ['empty', 'not_empty', 'today'].indexOf(condition[1]) === -1 ? condition[1] : true
    )

    return (form$, Validator, el$) => {
      var actual = get(form$.requestData, field)
      var expected = value

      return compare(actual, operator, expected, this.element$)
    }
  }
}
 
export default Factory
